/* ######################################################################### */
/* ##                                                                     ## */
/* ##  D B T U T O O L S / C O R E / D B 2 / B I N D F I L E . J A V A    ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Job .......: Wrapper class, implementing DB2 bind file             ## */
/* ##               related stuff.                                        ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Copyright (C) 2002  Daniel Scheibli                                ## */
/* ##                                                                     ## */
/* ##  This library is free software; you can redistribute it and/or      ## */
/* ##  modify it under the terms of the GNU Lesser General Public         ## */
/* ##  License as published by the Free Software Foundation; either       ## */
/* ##  version 2.1 of the License, or (at your option) any later version. ## */
/* ##                                                                     ## */
/* ##  This library is distributed in the hope that it will be useful,    ## */
/* ##  but WITHOUT ANY WARRANTY; without even the implied warranty of     ## */
/* ##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  ## */
/* ##  Lesser General Public License for more details.                    ## */
/* ##                                                                     ## */
/* ##  You should have received a copy of the GNU Lesser General Public   ## */
/* ##  License along with this library; if not, write to the Free         ## */
/* ##  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,     ## */
/* ##  MA 02111-1307 USA                                                  ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Remark ....: <none>                                                ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Author ....: Daniel Scheibli <daniel.scheibli@edelbyte.org>        ## */
/* ##  Date ......: 2002-08-10                                            ## */
/* ##  Changes ...: ----- BindFile.java V00.99/R020 --------------------- ## */
/* ##               <none>                                                ## */
/* ##               ----- BindFile.java V00.99/R011 --------------------- ## */
/* ##               2002-10-23 - Method toTimestamp() can through the     ## */
/* ##                            NullPointerException exception too.      ## */
/* ##               2002-10-23 - Added private constructor to prevent     ## */
/* ##                            the instantiation of this call.          ## */
/* ##               2002-10-16 - Added comments for javadoc.              ## */
/* ##               2002-10-09 - Renamed the class and package (was       ## */
/* ##                            org.edelbyte.DbtuTools.DB2BindFile).     ## */
/* ##               2002-09-01 - Method toTimestamp() throughs another    ## */
/* ##                            exception now (was NumberFormatExcept.)  ## */
/* ##               ----- BindFile.java V00.99/R010 --------------------- ## */
/* ##               <none>                                                ## */
/* ##                                                                     ## */
/* ######################################################################### */



/* ##### Presettings */
/* ------------------------------------------------------------------------- */
package org.dbtutools.DbtuTools.core.db2;
/* ------------------------------------------------------------------------- */





/* ##### Class definition */
/** ************************************************************************
 * Wrapper class, implementing DB2 bindfile related stuff.
 * It cannot be instantiated.
 * @author Daniel Scheibli &lt;daniel.scheibli at edelbyte.org&gt;
 * ************************************************************************* */
public class BindFile
{
 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   B I N D F I L E ()                                               ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Private declared Constructor, preventing the instantiation
  * of the <code>BindFile</code> class.
  * ************************************************************************ */
 private BindFile( )
 { }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   T O T I M E S T A M P ()                                         ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Converts a given Consistency Token into a timestamp.<br />
  * <br />
  * <i>Background:</i> The Consistency Token is a unique identifier,
  * which is used to ensure, the application and bindfile/package
  * are based on the same build.
  * @param oConsistencyToken
  *        The String holding the Consistency Token value.
  * @return A new String, representing the calculated timestamp.
  * @exception java.lang.IllegalArgumentException
  *            For the case, that the given Consistency Token has an
  *            invalid format, contains unexpected characters or the
  *            result is out of range.
  * @exception java.lang.NullPointerException
  *            If the <code>oConsistencyToken</code> argument was
  *            <code>null</null>.    
  * ************************************************************************ */
 public static java.lang.String toTimestamp( java.lang.String oConsistencyToken ) throws java.lang.IllegalArgumentException,
                                                                                         java.lang.NullPointerException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */
  char[]                  acConsistencyToken = oConsistencyToken.toCharArray();
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  java.lang.String        oConversionTable   = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890";
  char[]                  acConversionTable  = oConversionTable.toCharArray();
  java.text.DecimalFormat oFormatObject      = new java.text.DecimalFormat();
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  int[]                   aiTimestampToken   = new int[8];  
  java.lang.StringBuffer  oTimestamp;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  int                     iDummy;
  int                     I, J;
  /* ----------------------------------------------------------------------- */  



  /* ##### Check the given parameters */
  /* ----------------------------------------------------------------------- */
  if( oConsistencyToken == null )
  { throw new java.lang.NullPointerException(); }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  if( oConsistencyToken.length() != 8 ) 
  { throw new java.lang.IllegalArgumentException( "Consistency token must be 8 characters long." ); }
  /* ----------------------------------------------------------------------- */



  /* ##### Tokenize the Consistency Token, convert and check it */
  /* ------------------------------------------------------------------------ */
  for( I = 0; I < 8; I++ )
  {
   aiTimestampToken[I] = -1;
   for( J = 0; J < oConversionTable.length(); J++ )
   {
    if( acConsistencyToken[I] == acConversionTable[J] )
    { aiTimestampToken[I] = J; break; }
   }
   if( aiTimestampToken[I] == -1 )
   { throw new java.lang.IllegalArgumentException( "Consistency token must be a valid alphanumeric string." ); }
  }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  /* ##### If conversion results in more than 2 digits for milliseconds,... */  
  if( (aiTimestampToken[1] * oConversionTable.length() + aiTimestampToken[0]) >= 100 )
  { throw new java.lang.IllegalArgumentException( "Milliseconds part of the Consistency Token is not valid." ); }
  /* ----------------------------------------------------------------------- */  



  /* ##### Build and return the result string (in timestamp format) */
  /* ----------------------------------------------------------------------- */
  oTimestamp = new java.lang.StringBuffer( "" );
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  oFormatObject.applyPattern("0000");
  oTimestamp.append(oFormatObject.format(aiTimestampToken[7]+1984)).append("-");
  oFormatObject.applyPattern("00"); 
  oTimestamp.append(oFormatObject.format(aiTimestampToken[6])).append("-");
  oTimestamp.append(oFormatObject.format(aiTimestampToken[5])).append("-");
  oTimestamp.append(oFormatObject.format(aiTimestampToken[4])).append(".");  
  oTimestamp.append(oFormatObject.format(aiTimestampToken[3])).append(".");    
  oTimestamp.append(oFormatObject.format(aiTimestampToken[2])).append(".");
  oTimestamp.append(oFormatObject.format((aiTimestampToken[1] * oConversionTable.length() + aiTimestampToken[0]))).append("0000");
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */    
  return( oTimestamp.toString() );
  /* ----------------------------------------------------------------------- */  
 }			    
}
/* ------------------------------------------------------------------------- */




